@@ -18,26 +18,6 @@ public:
  * and otherwise it will yield an approximation
  */
 
-struct nlopt_return_value_msg {
-  nlopt_return_value_msg();
-  nlopt_return_value_msg(const int);
-
-  int nlopt_result_code;
-  bool is_error;
-  std::string message;
-};
-
-struct nlopt_return_value_msgs {
-  std::map<int, nlopt_return_value_msg> msgs;
-  bool any_errors = false;
-
-public:
-  void insert(const nlopt_return_value_msgs&);
-  void insert(const nlopt_return_value_msg&&);
-  bool has_any_errors() const;
-  std::string message() const;
-};
-
 class cdist_comb_generator {
   std::vector<PF_cdist*> &cdists;
   arma::mat neg_K;
@@ -45,7 +25,6 @@ class cdist_comb_generator {
   arma::vec k;
   /* less than 2 implies multivariate t-distribution */
   const int nu;
-  int nlopt_result_code;
 
 public:
   cdist_comb_generator
@@ -57,8 +36,6 @@ public:
      const arma::mat *xtra_covar = nullptr, const double covar_fac = -1,
      const double ftol_rel = 1e-6);
 
-  nlopt_return_value_msg get_result_code();
-
   std::unique_ptr<dist_comb> get_dist_comb(
       const std::initializer_list<arma::vec*>&);
 };
